@@ -42,8 +42,14 @@
 #include "newsreader.h"
 #include "config_gui_callbacks.h"
 
+/* $B%j%9%HJ]B8$K$O(BXBEL$B$r;HMQ(B */
+#undef USE_OPML
+
 #ifdef DEBUG
-#define DEBUG_PRINT(...) g_message(__VA_ARGS__)
+static pthread_mutex_t debugprint_mutex = PTHREAD_MUTEX_INITIALIZER;
+#define DEBUG_PRINT(...) pthread_mutex_lock(&debugprint_mutex); \
+                         g_message(__VA_ARGS__); \
+                         pthread_mutex_unlock(&debugprint_mutex);
 #else
 #define DEBUG_PRINT(...)
 #endif
@@ -73,6 +79,7 @@ static const int TOOLTIPS_LEN = 256;
 
 #define BORDER 6
 
+static pthread_mutex_t newsreader_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t getfeedlist_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t getfeed_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t msgdisp_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -105,12 +112,57 @@ free_feeddata(feeddata_t* pFeedData)
 	return;
 }
 
+static GList*
+copy_outlineList(newsreader_t* newsreader)
+{
+	GList* pList = NULL;
+	GList* pOutlineList = NULL;
+	outline_t* pOutlineSrc;
+	outline_t* pOutlineDst;
+
+	for(pList = newsreader->poOutlineList; pList; pList = pList->next){
+		if(!(pOutlineSrc = (outline_t*)pList->data)){
+			DEBUG_PRINT("copy_outlinelist fatal error");
+			return NULL;
+		}
+		pOutlineDst = g_new0(outline_t,1);
+		if(pOutlineSrc->strTitle)
+			pOutlineDst->strTitle = g_string_new(pOutlineSrc->strTitle->str);
+		if(pOutlineSrc->strURL)
+			pOutlineDst->strURL = g_string_new(pOutlineSrc->strURL->str);
+		pOutlineDst->bAvailable = pOutlineSrc->bAvailable;
+		pOutlineDst->iValidHours = pOutlineSrc->iValidHours;
+		pOutlineList = g_list_append(pOutlineList, pOutlineDst);
+	}	
+	
+	return pOutlineList;
+}
+
+static void
+free_outlineList(GList* pOutlineList)
+{
+	GList* pList;
+	outline_t* pOutline;
+	
+	for(pList = pOutlineList; pList; pList = pList->next){
+		if(!(pOutline = (outline_t*)pList->data))
+			continue;
+		if(pOutline->strTitle)
+			g_string_free(pOutline->strTitle, TRUE);
+		if(pOutline->strURL)
+			g_string_free(pOutline->strURL, TRUE);
+		g_free(pOutline);
+	}
+
+	g_list_free(pOutlineList);
+}
+
 static void
 StoreFeedList2File(newsreader_t* newsreader)
 {
 	gchar* filename;
 
-#if 0
+#ifdef USE_OPML
 	/* OPML$B$GJ]B8(B */
 	filename = xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
 											"xfce4" G_DIR_SEPARATOR_S "newsreader.xml",
@@ -118,7 +170,7 @@ StoreFeedList2File(newsreader_t* newsreader)
 
 	storeOutlineList(newsreader->poOutlineList, filename);
 	g_free(filename); 
-#endif
+#else
 
 	/* XBEL$B$GJ]B8(B */
 	filename = xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
@@ -126,6 +178,7 @@ StoreFeedList2File(newsreader_t* newsreader)
 											TRUE);
 	storeFeedList_XBEL(newsreader->poOutlineList, filename);
 	g_free(filename);
+#endif
 
 	return;
 }
@@ -216,6 +269,75 @@ notify_uri(const char* web_command, const char* uri)
 	return;
 }
 
+/* $B%a%C%;!<%8I=<(4X?t(B (gtk$B$N%a%$%s%k!<%W$+$i8F$P$l$k(B) */
+static gboolean
+msg_display_func(newsreader_t* newsreader)
+{
+	gchar* pchMsg = NULL;
+	GList* pList;
+	msgdata_t* pMsgdata;
+	GtkWidget *dialog;
+
+	pthread_mutex_lock(&msgdisp_mutex);
+		
+	while((pList = newsreader->pMsgList)){
+		if((pMsgdata = (msgdata_t*)pList->data)){
+			if(pMsgdata->pchMsg){
+				pchMsg = g_strdup(pMsgdata->pchMsg);
+				g_free(pMsgdata->pchMsg);
+			}
+			else{
+				DEBUG_PRINT("pMsgdata->pchMsg is null");
+			}
+			newsreader->pMsgList = g_list_remove(newsreader->pMsgList, pMsgdata);
+			g_free(pMsgdata);
+		}
+		else{
+			DEBUG_PRINT("??? pMsgdata is null");
+		}
+		
+		DEBUG_PRINT(pchMsg);
+		
+		dialog = gtk_message_dialog_new (NULL,
+										 0, //GTK_DIALOG_DESTROY_WITH_PARENT,
+										 GTK_MESSAGE_WARNING,
+										 GTK_BUTTONS_CLOSE,
+										 (const gchar*)pchMsg);
+
+		/* Destroy the dialog when the user responds to it (e.g. clicks a button) */
+		g_signal_connect_swapped (dialog, "response",
+								  G_CALLBACK (gtk_widget_destroy),
+								  dialog);
+
+		gtk_widget_show_all (dialog);
+		g_free(pchMsg);		
+	}
+	
+	pthread_mutex_unlock(&msgdisp_mutex);
+
+	/* FALSE$B$rJV$7$F=*N;$HF1;~$K%$%Y%s%H%k!<%W$h$j:o=|$7$F$b$i$&(B */
+	return FALSE;
+}
+
+/* $B%$%Y%s%H%k!<%W$K%a%C%;!<%8%@%$%"%m%0I=<(4X?t$r%"%$%I%k%=!<%9$H$7$FEPO?(B */
+static void
+message_register(newsreader_t* newsreader, const gchar* pchMsg)
+{
+	msgdata_t *pMsgdata;
+
+	pMsgdata = g_new0(msgdata_t, 1);
+	pMsgdata->pchMsg = g_strdup(pchMsg);
+	pthread_mutex_lock(&msgdisp_mutex);
+	newsreader->pMsgList = g_list_append(newsreader->pMsgList, pMsgdata);
+	pthread_mutex_unlock(&msgdisp_mutex);
+
+	/* msg_display_func$B$O(B1$B2s$7$+<B9T$5$l$J$$$+$i(B
+	   $B9b$a$NM%@hEY$G$bLdBj$J$$(B */
+	g_idle_add_full(G_PRIORITY_HIGH_IDLE, (GSourceFunc)msg_display_func, newsreader, NULL);
+
+	return;
+}
+
 /* $B%]%C%W%"%C%W%a%K%e!<MQ%j%9%H$N%/%j%"(B */
 static void
 ClearFeedListMenu(newsreader_t* newsreader)
@@ -632,25 +754,31 @@ GetFeedList(newsreader_t* newsreader)
 	GList* poFeedList = NULL;
 	GList* poNewList = NULL;
 	GList* poOldList = NULL;
-	GList* pOutlineList = newsreader->poOutlineList;
+	GList* pOutlineList = NULL;
 	/*static GStaticMutex newsreader_feedlist_mutex = G_STATIC_MUTEX_INIT;*/
 
-	DEBUG_PRINT("GetFeedList Start");
-
 	/*	if(!(g_static_mutex_trylock (&newsreader_feedlist_mutex)))
 		return NULL;*/
-	if(pthread_mutex_trylock(&getfeedlist_mutex))
+	if(pthread_mutex_trylock(&getfeedlist_mutex)){
+		DEBUG_PRINT("GetFeedList Locked");
 		return NULL;
+	}
+	
+	DEBUG_PRINT("GetFeedList Start");
 
 	pthread_mutex_lock(&getfeed_mutex);
 
+	pthread_mutex_lock(&newsreader_mutex);
+	DEBUG_PRINT("copy_outlineList");
+	pOutlineList = copy_outlineList(newsreader);
+	pthread_mutex_unlock(&newsreader_mutex);
+
 	poNewList = g_list_append(poNewList, CreateStaticFeed());
 
 	feedNo++;
 	outline_t* poOutline;
 
 	for(plist = pOutlineList; plist; plist = plist->next){
-
 		poOutline = (outline_t*)plist->data;
 		if(!poOutline || !poOutline->bAvailable)
 			continue;
@@ -705,10 +833,11 @@ GetFeedList(newsreader_t* newsreader)
 		
 		poNewList = g_list_append(poNewList, pNewFeedData);
 		feedNo++;
-		
 	}
 
-	XFCE_PANEL_LOCK();
+	free_outlineList(pOutlineList);
+
+	pthread_mutex_lock(&newsreader_mutex);
 
 	newsreader->g_pCurFeed = NULL;
 	newsreader->g_pCurItem = NULL;
@@ -716,8 +845,6 @@ GetFeedList(newsreader_t* newsreader)
 	poOldList = newsreader->g_pFeedList;
 	newsreader->g_pFeedList = poNewList;
 
-	XFCE_PANEL_UNLOCK();
-
 	for(poFeedList = poOldList; poFeedList; poFeedList = poFeedList->next){
 		for(plist = newsreader->g_pFeedList; plist; plist = plist->next){
 			if(poFeedList->data != NULL &&
@@ -734,6 +861,8 @@ GetFeedList(newsreader_t* newsreader)
 
 	UpdateFeedListMenu(newsreader);
 
+	pthread_mutex_unlock(&newsreader_mutex);
+
 	/*	g_static_mutex_unlock (&newsreader_feedlist_mutex);*/
 	pthread_mutex_unlock(&getfeed_mutex);
 	pthread_mutex_unlock(&getfeedlist_mutex);
@@ -747,6 +876,9 @@ static gboolean SetUpdateFeedTimer(void *p_newsreader)
   struct newsreader_t *newsreader = (newsreader_t*) p_newsreader;
   pthread_t thread;
 
+  DEBUG_PRINT("SetUpdateFeedTimer start");
+  pthread_mutex_lock(&newsreader_mutex);
+
   if (newsreader->iFeedTimerID) {
 	g_source_remove (newsreader->iFeedTimerID);
 	newsreader->iFeedTimerID = 0;
@@ -758,6 +890,9 @@ static gboolean SetUpdateFeedTimer(void *p_newsreader)
   newsreader->iFeedTimerID = g_timeout_add (newsreader->feedUpdateMin * 60000,
 												  (GtkFunction) SetUpdateFeedTimer, newsreader);
 
+  pthread_mutex_unlock(&newsreader_mutex);
+  DEBUG_PRINT("SetUpdateFeedTimer end");
+
   return (1);
 
 }
@@ -775,13 +910,13 @@ static gboolean SetTimer (void *p_newsreader)
 
 	//    struct param_t *poConf = &(poPlugin->oConf.oParam);
 
+	pthread_mutex_lock(&newsreader_mutex);
+
     if (newsreader->iTimerID) {
 		g_source_remove (newsreader->iTimerID);
 		newsreader->iTimerID = 0;
     }
 
-	XFCE_PANEL_LOCK();
-
 	tmCur = time(NULL);
 
 	for(;;){
@@ -839,43 +974,18 @@ static gboolean SetTimer (void *p_newsreader)
 	}
 	else{
 		gtk_image_set_from_stock(newsreader->wIcon, "gtk-dialog-info", GTK_ICON_SIZE_MENU);	
-	}
-	
-	/*	GList* poList = newsreader->poIconList;
-		feed_t* pFeed = ((feeddata_t*)(newsreader->g_pCurFeed->data))->pFeed;
-
-		for(; poList; poList = poList->next){
-		favicon_t* icon = (favicon_t*)poList->data;
-		if(!icon)
-		continue;
-		if(strcmp(icon->uri, pFeed->url->str)==0){
-		if(icon->pIconPixbuf){
-		gtk_image_set_from_pixbuf(newsreader->wIcon, gdk_pixbuf_scale_simple(icon->pIconPixbuf, 16, 16, GDK_INTERP_BILINEAR));
-		}
-		else{
-		gtk_image_set_from_stock(newsreader->wIcon, "gtk-dialog-info", GTK_ICON_SIZE_MENU);
-		}
-			
-		break;
-		}
-
-		}
-
-		if(!poList){
-		gtk_image_set_from_stock(newsreader->wIcon, "gtk-dialog-info", GTK_ICON_SIZE_MENU);
-		}
-	*/
+	}	
 
 	/* set tooltips */
 	gtk_tooltips_set_tip(newsreader->wTooltips, GTK_WIDGET(newsreader->wEvItem), buff, "");
 
-	XFCE_PANEL_UNLOCK();
-
 	gtk_label_set_text(newsreader->wLblItem, buff);
 	
     newsreader->iTimerID = g_timeout_add (newsreader->itemUpdateSec * 1000,
 										  (GtkFunction) SetTimer, newsreader);
 
+	pthread_mutex_unlock(&newsreader_mutex);
+
     return (1);
 }				/* SetTimer() */
 
@@ -894,6 +1004,8 @@ newsreader_read_config (Control * control, xmlNodePtr node)
     newsreader_t *newsreader;
 	PangoFontDescription* pFontDesc = NULL;
 
+	DEBUG_PRINT("newsreader_read_config start");
+
 	newsreader = (newsreader_t *) control->data;
 
     if (!node){
@@ -985,7 +1097,10 @@ newsreader_read_config (Control * control, xmlNodePtr node)
 	
 	/* feed $BFI$_9~$_$N%?%$%^!<:F@_Dj(B */
 	SetUpdateFeedTimer( newsreader );
-	
+
+	DEBUG_PRINT("newsreader_read_config end");
+
+	return;
     /* do stuff to the widget or otherwise apply new settings */
 }
 
@@ -1022,7 +1137,9 @@ newsreader_write_config (Control * control, xmlNodePtr parent)
 	if(newsreader->bUseTooltips)
 		xmlSetProp (parent, "newsreader_Tooltips", "true");
 
+	pthread_mutex_lock(&newsreader_mutex);
 	StoreFeedList2File(newsreader);
+	pthread_mutex_unlock(&newsreader_mutex);
 
 	return;
 }
@@ -1096,16 +1213,16 @@ on_button_release_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
 		return TRUE;
 	}
 			
-	XFCE_PANEL_LOCK ();
+	pthread_mutex_lock (&newsreader_mutex);
 	if(!((item_t*)newsreader->g_pCurItem->data)->url->str ||
 	   !(*((item_t*)newsreader->g_pCurItem->data)->url->str)){
-		XFCE_PANEL_UNLOCK ();
+		pthread_mutex_unlock(&newsreader_mutex);
 		return TRUE;
 	}
 
 	notify_uri(newsreader->pchBrowserCmd, ((item_t*)newsreader->g_pCurItem->data)->url->str);
 
-	XFCE_PANEL_UNLOCK ();
+	pthread_mutex_unlock(&newsreader_mutex);
 
 	return TRUE;
   
@@ -1146,61 +1263,6 @@ on_leave_notify_cb (GtkEventBox *widget, GdkEventCrossing *event, newsreader_t*
   return TRUE;
 }
 
-/* $B%a%C%;!<%8I=<(4X?t(B */
-static gboolean
-msg_display_func(newsreader_t* newsreader)
-{
-	gchar* pchMsg = NULL;
-	GList* pList;
-	msgdata_t* pMsgdata;
-	GtkWidget *dialog;
-
-	pthread_mutex_lock(&msgdisp_mutex);
-		
-	while((pList = newsreader->pMsgList)){
-		if((pMsgdata = (msgdata_t*)pList->data)){
-			if(pMsgdata->pchMsg){
-				pchMsg = g_strdup(pMsgdata->pchMsg);
-				g_free(pMsgdata->pchMsg);
-			}
-			else{
-				DEBUG_PRINT("pMsgdata->pchMsg is null");
-			}
-			newsreader->pMsgList = g_list_remove(newsreader->pMsgList, pMsgdata);
-			g_free(pMsgdata);
-		}
-		else{
-			DEBUG_PRINT("??? pMsgdata is null");
-		}
-		
-		/*xfce_warn(pchMsg);*/
-		DEBUG_PRINT(pchMsg);
-
-		/* FIXME */
-		/* $B0J2<$N%(%i!<$,$G$k$?$a%3%a%s%H%"%&%H(B */
-		/* Xlib: unexpected async reply (sequence 0x178a)!*/
-		/* (xfce4-panel:27398): Gdk-CRITICAL **: gdk_window_set_geometry_hints: assertion `window != NULL' failed*/
-		
-		dialog = gtk_message_dialog_new (NULL,
-										 0, //GTK_DIALOG_DESTROY_WITH_PARENT,
-										 GTK_MESSAGE_WARNING,
-										 GTK_BUTTONS_CLOSE,
-										 (const gchar*)pchMsg);
-
-		/* Destroy the dialog when the user responds to it (e.g. clicks a button) */
-		g_signal_connect_swapped (dialog, "response",
-								  G_CALLBACK (gtk_widget_destroy),
-								  dialog);
-
-		gtk_widget_show_all (dialog);
-		g_free(pchMsg);		
-	}
-	
-	pthread_mutex_unlock(&msgdisp_mutex);
-
-	return FALSE;
-}
-
 /**
  * newsreader_new
  *
@@ -1281,12 +1343,13 @@ newsreader_new (void)
 		doc = xmlParseFile(filename);
 		if (doc) {
 			DEBUG_PRINT("get feedlist from file: %s", filename);
-			XFCE_PANEL_LOCK();
-#if 0
+			pthread_mutex_lock(&newsreader_mutex);
+#ifdef USE_OPML
 			newsreader->poOutlineList =  getOutlineList(doc, newsreader->poOutlineList);
-#endif
+#else
 			newsreader->poOutlineList =  getFeedList_XBEL(doc, newsreader->poOutlineList);
-			XFCE_PANEL_UNLOCK();
+#endif
+			pthread_mutex_unlock(&newsreader_mutex);
 		}
 		else{
 			g_warning("parse error: %s", filename);
@@ -1313,14 +1376,6 @@ newsreader_free (Control * control)
 {
     newsreader_t *newsreader = (newsreader_t *) control->data;
 
-	/* stop msg_display_func thread */
-	/*	pthread_mutex_lock(&msgdisp_mutex);
-	newsreader->active = FALSE;
-	pthread_mutex_unlock(&msgdisp_mutex);
-	pthread_cond_signal(&msgdisp_cond);
-	pthread_join(newsreader->msgdisp_tid, NULL);
-	*/
-
     /* free resources, stop timeouts, etc. */
 
 	if(newsreader->iTimerID)
@@ -1382,23 +1437,6 @@ free_newsreader_dialog (NewsreaderDialog * rd)
 }
 #endif
 
-static void
-free_outlineList(newsreader_t* newsreader)
-{
-	GList* pList;
-	outline_t* pfeed;
-	
-	for(pList = newsreader->poOutlineList; pList; pList = pList->next){
-		pfeed = (outline_t*)pList->data;
-		g_string_free(pfeed->strTitle, TRUE);
-		g_string_free(pfeed->strURL, TRUE);
-		g_free(pfeed);
-	}
-
-	g_list_free(newsreader->poOutlineList);
-	newsreader->poOutlineList = NULL;
-}
-
 /**
  * newsreader_apply_options
  *
@@ -1430,7 +1468,6 @@ newsreader_apply_options (newsreader_t * newsreader)
 	//    gtk_widget_set_size_request(newsreader->widget, newsreader->labelWidth, -1);
 	gtk_widget_set_size_request(newsreader->control->base, newsreader->labelWidth, -1);
 
-
 	if(newsreader->strItemFontName)
 		g_string_free(newsreader->strItemFontName,TRUE);
 
@@ -1453,8 +1490,10 @@ newsreader_apply_options (newsreader_t * newsreader)
 	gboolean valid;
 	GtkTreeIter iter;
 
-	XFCE_PANEL_LOCK ();
-	free_outlineList(newsreader);
+	pthread_mutex_lock(&newsreader_mutex);
+
+	free_outlineList(newsreader->poOutlineList);
+	newsreader->poOutlineList = NULL;
 	
 	valid = gtk_tree_model_get_iter_first(newsreader->oGUI.wModel, &iter);
 
@@ -1486,7 +1525,7 @@ newsreader_apply_options (newsreader_t * newsreader)
 		valid = gtk_tree_model_iter_next(newsreader->oGUI.wModel, &iter);
 	}
 	
-	XFCE_PANEL_UNLOCK ();
+	pthread_mutex_unlock(&newsreader_mutex);
 
 	if(bFeedUpdate)
 		SetUpdateFeedTimer( newsreader );
@@ -1536,6 +1575,8 @@ set_model_data(newsreader_t* newsreader)
 	GList* poList;
 	GtkTreeIter iter;
 
+	pthread_mutex_lock(&newsreader_mutex);
+
 	for(poList = newsreader->poOutlineList; poList; poList = poList->next){
 		poOutline = (outline_t*)poList->data;
 		gtk_list_store_append (GTK_LIST_STORE(newsreader->oGUI.wModel), &iter);
@@ -1546,6 +1587,8 @@ set_model_data(newsreader_t* newsreader)
 							COL_UNTIL, poOutline->iValidHours,
 							-1);
 	}
+
+	pthread_mutex_unlock(&newsreader_mutex);
 }
 
 static void
@@ -1686,9 +1729,13 @@ create_initial_menu(newsreader_t* newsreader)
 	GtkWidget* itemSeparator;
 	GtkWidget* itemFeedList;
 	GtkWidget* itemSubMenu;
+	GtkWidget* itemAbout;
+	GtkWidget* itemImage;
 
 	menu = gtk_menu_new();
-	itemUpdate =  gtk_menu_item_new_with_mnemonic (_("_Update"));
+	itemImage = gtk_image_new_from_stock("gtk-refresh", GTK_ICON_SIZE_MENU);
+	itemUpdate = gtk_image_menu_item_new_with_mnemonic (_("_Update"));
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(itemUpdate), itemImage);
 	gtk_widget_show(itemUpdate);
 	gtk_menu_append(menu, itemUpdate);
 	newsreader->wItmUpdate = GTK_MENU_ITEM(itemUpdate);
@@ -1714,11 +1761,15 @@ create_initial_menu(newsreader_t* newsreader)
 	gtk_widget_show(itemSeparator);
 	gtk_menu_append(menu, itemSeparator);
 
-	itemUpdate =  gtk_menu_item_new_with_mnemonic (_("_About"));
-	gtk_widget_show(itemUpdate);
-	gtk_menu_append(menu, itemUpdate);
-	newsreader->wItmUpdate = GTK_MENU_ITEM(itemUpdate);
-	g_signal_connect (G_OBJECT(newsreader->wItmUpdate), "activate",
+
+	itemImage = gtk_image_new_from_stock("gtk-dialog-info", GTK_ICON_SIZE_MENU);
+	itemAbout = gtk_image_menu_item_new_with_mnemonic (_("_About"));
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(itemAbout), GTK_WIDGET(itemImage));
+
+	gtk_widget_show(itemAbout);
+	gtk_menu_append(menu, itemAbout);
+	newsreader->wItmAbout = GTK_MENU_ITEM(itemAbout);
+	g_signal_connect (G_OBJECT(newsreader->wItmAbout), "activate",
 					  G_CALLBACK(About), newsreader);
 
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(itemFeedList), submenu);
@@ -1735,7 +1786,6 @@ dragedfeed_append_to_list(newsreader_t* newsreader)
 	feed_t* pFeed = NULL;
 	feeddata_t* pFeedData = NULL;
 	outline_t* pOutline = NULL;
-	msgdata_t* pMsgdata;
 
 	pthread_mutex_lock(&getfeed_mutex);
 
@@ -1761,7 +1811,7 @@ dragedfeed_append_to_list(newsreader_t* newsreader)
 
 	AppendIconList(newsreader, pFeedData);
 
-	XFCE_PANEL_LOCK();
+	pthread_mutex_lock(&newsreader_mutex);
 
 	pOutline = g_new0(outline_t, 1);
 	pOutline->strTitle = g_string_new(pFeed->title->str);
@@ -1777,25 +1827,19 @@ dragedfeed_append_to_list(newsreader_t* newsreader)
 
 	UpdateFeedListMenu(newsreader);
 
-	XFCE_PANEL_UNLOCK();
-
 	g_free(newsreader->pchURI);
 
 	StoreFeedList2File(newsreader);
 
+	pthread_mutex_unlock(&newsreader_mutex);
+
 	pthread_mutex_unlock(&getfeed_mutex);
 
 	return;
 
  error:
 	pthread_mutex_unlock(&getfeed_mutex);
-
-	pMsgdata = g_new0(msgdata_t, 1);
-	pMsgdata->pchMsg = g_strdup("Could not Acquire RSS/Atom feed");
-	pthread_mutex_lock(&msgdisp_mutex);
-	newsreader->pMsgList = g_list_append(newsreader->pMsgList, pMsgdata);
-	pthread_mutex_unlock(&msgdisp_mutex);
-	g_idle_add((GSourceFunc)msg_display_func, newsreader);
+	message_register(newsreader, "Could not Acquire RSS/Atom feed");
 	return;
 }
 
@@ -1818,7 +1862,6 @@ void on_wLblItem_drag_drop( GtkWidget        *widget,
 	gchar* pchData = NULL;
 	gchar** ppchSplit = NULL;
 	pthread_t thread;
-	msgdata_t* pMsgdata;
 
 	if ((data->length <= 0) || (data->format != 8)) {
 		g_warning("drag have no data");
@@ -1868,12 +1911,7 @@ void on_wLblItem_drag_drop( GtkWidget        *widget,
 
  error:
 	gtk_drag_finish(context, FALSE, TRUE, time_);
-	pMsgdata = g_new0(msgdata_t, 1);
-	pMsgdata->pchMsg = g_strdup("Could not Acquire RSS/Atom feed");
-	pthread_mutex_lock(&msgdisp_mutex);
-	newsreader->pMsgList = g_list_append(newsreader->pMsgList, pMsgdata);
-	pthread_mutex_unlock(&msgdisp_mutex);
-	g_idle_add((GSourceFunc)msg_display_func, newsreader);
+	message_register(newsreader, "Could not Acquire RSS/Atom feed");
 	return;
 }
 
@@ -1954,7 +1992,6 @@ create_newsreader_control (Control * control)
 G_MODULE_EXPORT void
 xfce_control_class_init (ControlClass * cc)
 {
-
     xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
     /* Must be present: 
@@ -1964,7 +2001,7 @@ xfce_control_class_init (ControlClass * cc)
        - attach_callback : connect a signal (e.g. right-click menu)
        - free            : free allocated resources of Control instance
 	*/
-    cc->name = "newsreader";
+    cc->name = "Newsreader";
     cc->caption = "Newsreader";
 
     cc->create_control = (CreateControlFunc) create_newsreader_control;
